@@ -20,10 +20,15 @@ def log_subprocess_output(pipe):
     for line in iter(pipe.readline, b''): # b'\n'-separated lines
         logging.info(line.decode()[0:-1])
 
+#original
 #shell_cmd = os.environ['HOME'] + '/Soft/timingsuite/dist/Debug/GNU-MacOSX/timingsuite <timing_cmd.txt'
+
+
 shell_cmd = 'timingsuite <timing_cmd.txt'
 
 
+
+
 def run(cmd=shell_cmd):
     logger.info("------------------------------------------------------------------------------------------------\n")
     logger.info("**** running %s ****\n" % cmd)
@@ -119,6 +124,7 @@ def plot_periodogram():
     from IPython.display import Image
     from IPython.display import display
     _ = display(Image(filename='tmp.gif', format="gif"))
+    return
 
 
 efold_cmd='''14
@@ -342,10 +348,6 @@ no
 
 '''
 
-#event_file_name  gti_file_name  output_event_file_name
-
-
-
 def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile=None, nudot=0, t_ref=0, user_gti=None):
     
     '''
@@ -472,8 +474,10 @@ def pad_matrix_with_zeros(x1_min, x1_max, pp1, dpp1, x2_min, x2_max):
     return np.array(new_x1_min), np.array(new_x1_max), np.array(new_pp1), np.array(new_dpp1)
 
 
-def read_one_matrix(kind, unit, background_difference_limit=5):
+def read_one_matrix(kind, unit, background_difference_limit=-1, use_counts=False, subtract_background=True):
     '''
+    Read matrix from one NuSTAR unit and optionally subtract the background.
+    Note that if background_difference_limit>0, the output source and background matrixes might have different shape
 
     :param kind: 'E' for Energy-Phase, 'T' for Time-Phase
     :param unit: 'A' or 'B' for FPM unit
@@ -481,10 +485,14 @@ def read_one_matrix(kind, unit, background_difference_limit=5):
             =0 does not subtract the background
             <0 subtracts the background after padding background matrix with zeros
             >0  if sizes of source and background matrices differs by more than this value, it will skip subtraction
+    :param use_counts: if correcting for the exposure
     :return: x_min lower edge of time or energy bins
              x_min upper edge of time or energy bins,
              pp matrix
              dpp matrix of uncertainties
+             pp_b background matrix
+             dpp_b matrix of background uncertainties
+             
     '''
     import astropy.io.fits as pf
 
@@ -509,47 +517,76 @@ def read_one_matrix(kind, unit, background_difference_limit=5):
     t_maxA = ff[1].data[key2]
     ptA = ff[1].data['MATRIX']
     dptA = ff[1].data['ERROR']
+    if 'Exposure' in ff[1].data.names:
+        exposureA = ff[1].data['Exposure']
+    else:
+        exposureA = np.ones(ptA.shape)
     ff.close()
+    if use_counts:
+        ptA *= exposureA
+        dptA *= exposureA
 
     ff = pf.open(fname_bck, 'readonly')
     t_minAb = ff[1].data[key1]
     t_maxAb = ff[1].data[key2]
     ptbA = ff[1].data['MATRIX']
     dptbA = ff[1].data['ERROR']
+    if 'Exposure' in ff[1].data.names:
+        exposure_bA = ff[1].data['Exposure']
+    else:
+        exposure_bA = np.ones(ptbA.shape)
     ff.close()
 
-    if background_difference_limit < 0:
-        t_minA, t_maxA, ptA, dptA, t_minAb, t_maxAb, ptbA, dptbA = \
-            pad_matrices_with_zeros(t_minA, t_maxA, ptA, dptA, t_minAb, t_maxAb, ptbA, dptbA)
-        ptA -= ptbA
-        dptA = np.sqrt(dptbA ** 2 + dptA ** 2)
-        logger.info('Subtracted the background, possibly padded with zeros')
-    elif background_difference_limit > 0:
-        if np.abs(len(t_minA) - len(t_minAb)) < background_difference_limit:
-            indA, indAb = get_ind_combine(t_minA, t_minAb)
-            t_minA = t_minA[indA]
-            t_maxA = t_maxA[indA]
+    if use_counts:
+        ptbA *= exposure_bA
+        dptbA *= exposure_bA
+    
+        if background_difference_limit < 0:
+            t_minA, t_maxA, ptA, dptA, t_minAb, t_maxAb, ptbA, dptbA = \
+                pad_matrices_with_zeros(t_minA, t_maxA, ptA, dptA, t_minAb, t_maxAb, ptbA, dptbA)
+            if subtract_background:
+                ptA -= ptbA
+                dptA = np.sqrt(dptbA ** 2 + dptA ** 2)
+                logger.info('Subtracted the background, possibly padded with zeros')
+            else:
+                logger.info("Not subtracting the background")
+        elif background_difference_limit > 0:
+            if np.abs(len(t_minA) - len(t_minAb)) < background_difference_limit and subtract_background:
+                indA, indAb = get_ind_combine(t_minA, t_minAb)
+                t_minA = t_minA[indA]
+                t_maxA = t_maxA[indA]
 
-            ptA = ptA[indA, :] - ptbA[indAb, :]
-            dptA = np.sqrt(dptA[indA, :] ** 2 + dptbA[indAb, :] ** 2)
-            logger.info('Subtracted the background')
-        else:
-            logger.info('We do not subtract the background because the difference in matrix dimension is %d' %
-                        np.abs(len(t_minA) - len(t_minAb)))
-    else:
-        logger.info("Not subtracting the background")
+                ptA = ptA[indA, :] - ptbA[indAb, :]
+                dptA = np.sqrt(dptA[indA, :] ** 2 + dptbA[indAb, :] ** 2)
+                logger.info('Subtracted the background')
+            else:
+                logger.info('We do not subtract the background: difference in matrix dimension is %d and subtract_backgroun is %b' %
+                            (np.abs(len(t_minA) - len(t_minAb)), subtract_background))
 
-    return t_minA, t_maxA, ptA, dptA
 
-def read_and_sum_matrixes(kind, background_difference_limit=5):
+    return t_minA, t_maxA, ptA, dptA, ptbA, dptbA
+
+def read_and_sum_matrixes(kind, background_difference_limit=-1, use_counts=False, subtract_background=True):
     '''
-
+    Reads and sum matrixes fromt he two NuSTAR units
+    (Note that if background_difference_limit>0, the method is not tested as the output source and background matrixes might have different shape)
+    
     :param kind: 'E' for ENERGY-Phase, 'T'  for 'TIME-Phase
-    :param background_difference_limit:
+    :param background_difference_limit: if >0 removes at most this number of rows to match background and source matrixes
+                                        if <=0 it pads the smalle matrix with zeros (default -1)
+    :param subtract background: boolean to subtract the background (default true)
     :return:
+    x_min array of minimum times or energies 
+    x_max array of maximum times or energies
+    pp matrix
+    dpp matrix uncertainties
+    pp_b backound matrix
+    dpp_b background matrix uncertainties
     '''
-    t_minA, t_maxA, ptA, dptA = read_one_matrix(kind, 'A', background_difference_limit=background_difference_limit)
-    t_minB, t_maxB, ptB, dptB = read_one_matrix(kind, 'B', background_difference_limit=background_difference_limit)
+    t_minA, t_maxA, ptA, dptA, ptbA, dptbA = read_one_matrix(kind, 'A', background_difference_limit=background_difference_limit, use_counts=use_counts
+                                                , subtract_background=subtract_background)
+    t_minB, t_maxB, ptB, dptB, ptbB, dptbB = read_one_matrix(kind, 'B', background_difference_limit=background_difference_limit, use_counts=use_counts
+                                                , subtract_background=subtract_background)
 
     if background_difference_limit > 0 :
         indA, indB = get_ind_combine(t_minA, t_minB)
@@ -557,18 +594,26 @@ def read_and_sum_matrixes(kind, background_difference_limit=5):
         t_max = t_maxA[indA]
         pp = ptA[indA, :] + ptB[indB, :]
         dpp = np.sqrt(dptA[indA, :] ** 2 + dptB[indB, :] ** 2)
+        pp_b = ptbA[indA, :] + ptbB[indB, :]
+        dpp_b = np.sqrt(dptbA[indA, :] ** 2 + dptbB[indB, :] ** 2)
+        
     else:
         t_min, t_max, ptA, dptA, t_minB, t_maxB, ptB, dptB = \
             pad_matrices_with_zeros(t_minA, t_maxA, ptA, dptA, t_minB, t_maxB, ptB, dptB)
-
+        _, _, ptbA, dptbA, t_minB, t_maxB, ptbB, dptbB = \
+            pad_matrices_with_zeros(t_minA, t_maxA, ptbA, dptbA, t_minB, t_maxB, ptbB, dptbB)
+        #print(ptA.shape, ptB.shape, ptbA.shape,  ptbB.shape)
         pp = ptA + ptB
         dpp = np.sqrt(dptA ** 2 + dptB ** 2)
+        pp_b = ptbA + ptbB
+        dpp_b = np.sqrt(dptbA ** 2 + dptbB ** 2)
+        
 
     logger.info('Matrix for unit A has size %d x %d ' % (ptA.shape[0], ptA.shape[1]))
     logger.info('Matrix for unit B has size %d x %d ' % (ptB.shape[0], ptB.shape[1]))
     logger.info('The combined matrix has size %d x %d' % (pp.shape[0], pp.shape[1]))
 
-    return t_min, t_max, pp, dpp
+    return t_min, t_max, pp, dpp, pp_b, dpp_b
 
 def get_ind_combine_engine(x1, x2):
     '''
@@ -690,7 +735,7 @@ def make_tphase(freq,  min_en=3., max_en=70., t_step=1000, n_bins=32, orbitfile=
             run()
             shutil.copy('%s%s_TPHASE.fits' % (tt, unit), '%s%s_TPHASE_%03d.fits' % (tt, unit, n_bins))
 
-def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=False):
+def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=False, use_counts=False, background=None):
     '''
 
     :param e_min: array with minimum energy of each bin
@@ -699,11 +744,14 @@ def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=Fa
     :param dpp: 2-d array with pulse profile uncertainties
     :param min_s_n: minimum S/N for rebin
     :param only_pulsed:  it subtracts the average value before rebinning
+    ;param background a couple (background, background_uncertainty)
     :return:
     new_e_mins rebinned array with minimum energy of each bin
     new_e_maxs rebinned array with maximum energy of each bin
     new_pulses rebinned 2-d array with pulse profiles
     dnew_pulses rebinned 2-d array with pulse profile uncertainties
+    (new_background rebinned 2-d array with background pulse profiles
+    dnew_background rebinned 2-d array with background pulse profiles uncertainties) - returned only is background is not None 
     '''
 
     new_pulses = []
@@ -715,9 +763,20 @@ def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=Fa
 
     pp = copy.deepcopy(pp_input)
     dpp = copy.deepcopy(dpp_input)
+
+    if background is not None:
+        pp_b = copy.deepcopy(background[0])
+        dpp_b = copy.deepcopy(background[1])
+
+        new_pulses_b = []
+        dnew_pulses_b = []
+
     while i1 < len(e_min) - 1:
         p1 = copy.copy(pp[i1, :])
         dp1 = copy.copy(dpp[i1, :])
+        if background is not None:
+            p1_b = copy.copy(pp_b[i1, :])
+            dp1_b = copy.copy(dpp_b[i1, :])
         logger.debug('%d' % i1)
         for i2 in range(i1 + 1, len(e_min)):
 
@@ -730,8 +789,19 @@ def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=Fa
             logger.debug(s_n)
             #print(np.sum(np.abs(p1)), np.sqrt(np.sum(dp1 ** 2)), s_n)
             if s_n >= min_s_n or i2 == len(e_min) - 1:
-                new_pulses.append(p1/float(i2-i1))
-                dnew_pulses.append(dp1/float(i2-i1))
+                if use_counts:
+                    new_pulses.append(p1)
+                    dnew_pulses.append(dp1)
+                else:
+                    new_pulses.append(p1/float(i2-i1))
+                    dnew_pulses.append(dp1/float(i2-i1))
+                if background is not None:
+                    if use_counts:
+                        new_pulses_b.append(p1_b)
+                        dnew_pulses_b.append(dp1_b)
+                    else:
+                        new_pulses_b.append(p1_b/float(i2-i1))
+                        dnew_pulses_b.append(dp1_b/float(i2-i1))
                 new_e_mins.append(e_min[i1])
                 new_e_maxs.append(e_max[i2 - 1])
                 logger.debug("Boom %f %d %d " % (s_n, i1, i2))
@@ -747,22 +817,63 @@ def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=Fa
 
                 p1 += pp[i2, :]
                 dp1 = np.sqrt(dp1 ** 2 + dpp[i2, :] ** 2)
+                if background is not None:
+                    p1_b += pp_b[i2, :]
+                    dp1_b = np.sqrt(dp1_b ** 2 + dpp_b[i2, :] ** 2)
+                    
                 # print("i2", i2)
                 # print(np.count_nonzero(p1), np.count_nonzero(pp[i2, :]))
                 #It may happen that 1 background photon (negative rate) makes a bin to zero
                 # if np.count_nonzero(p1) < old_nonzero:
                 #     print(p1)
                 #     print(pp[i2, :])
-                old_nonzero = np.count_nonzero(p1)
+                #old_nonzero = np.count_nonzero(p1)
 
 
     logger.info('We rebinned from %d to %d bins at a minimum S/N of %.1f' % (len(e_min), len(new_e_mins), min_s_n))
-    return np.array(new_e_mins), np.array(new_e_maxs), np.array(new_pulses), np.array(dnew_pulses)
+    # not sure this is a good practice
+    if background is None:
+        return np.array(new_e_mins), np.array(new_e_maxs), np.array(new_pulses), np.array(dnew_pulses)
+    else:
+        return np.array(new_e_mins), np.array(new_e_maxs), np.array(new_pulses), np.array(dnew_pulses), np.array(new_pulses_b), np.array(dnew_pulses_b)
+
+def pulsed_fraction_area(c, dc, background=None, background_error=None): 
+    """It computes the pulsed fraction with the Area method (ref. )
+    remember PF is a factor  of about 1.4 the other PFs
+
+    Args:
+        c (numpy array): the pulse profile
+        dc (numpy array): the pulse profile uncertainty [used for error computation]
+
+    Returns:
+       numpy float: the pulsed fraction
+    """
+    logger.debug('remember PF is a factor  of about 1.4 the other PFs')
+    a0 = 0
+    if background is not None and background_error is not None:
+        a0 = subtract_background(a0, background,background_error)
+    return np.sum(c-np.min(c)+a0)/np.sum(c+a0)
+
+def pulse_fraction_from_data_rms(counts, counts_err, n_harm=-1, background=None, background_error=None, plot=False, label='', verbose=False):
+    
+    """pulsed fractio computation from fixed number of harmonics
+        following Archibald et al (2014) that uses de Jager et al. (1986)
+        we use and optimal number of harmonics, from Archibald et al. (2014), ppendix A
+    Args:
+        counts (_type_): pulse profile
+        counts_err (_type_): pulse profile uncertainty
+        n_harm (int, optional): number of used harmonics. If <=0, it determines the optimal number. default -1
+
+    Returns:
+        numppy double: pulsed fraction
+    """   
 
-def pulse_fraction_from_data_rms(counts, counts_err, n_harm=3):
     a0 = np.mean(counts)
-    K = n_harm
+
     N = np.size(counts)
+    K = n_harm
+    if n_harm <= 0:
+        K = int(N/2)
 
     A = np.zeros(K)
     B = np.zeros(K)
@@ -805,19 +916,72 @@ def pulse_fraction_from_data_rms(counts, counts_err, n_harm=3):
         sigma_b[k] = (1. / (N ** 2)) * SIGMA_B
         k = k + 1
 
-    somma = a ** 2 + b ** 2
+    if n_harm <=0:
+        m4 = 4*np.arange(1,K+1)
+        cumulative_sum = np.zeros(K)
+        for k in range(1,K):
+            cumulative_sum[k] = np.sum( (a[0:k]/sigma_a[0:k])**2 + (b[0:k]/sigma_b[0:k])**2)
+        max_harm = np.argmax(cumulative_sum - m4)+1
+        logger.debug("We stop at %d harmonics" % max_harm)
+    else:
+        max_harm = n_harm
+    somma = a[0:max_harm] ** 2 + b[0:max_harm] ** 2
     # print(somma)
-    differenza = sigma_a ** 2 + sigma_b ** 2
+    differenza = sigma_a[0:max_harm] ** 2 + sigma_b[0:max_harm] ** 2
     bla = somma - differenza
     # print('diff: ',differenza)
+    logger.debug('Pre background average %f' % a0)
+    if background is not None and background_error is not None:
+        a0 = subtract_background(a0, background, background_error)
+    logger.debug('Post background average %f' % a0)
+
     PF_rms = np.sqrt(2 * sum(bla)) / a0
-    return (PF_rms)
+
+    if plot:
+        import matplotlib.pyplot as plt
+        #plt.figure()
+        f = np.linspace(0, 1, int(N))
+        if n_harm > 1:
+            plt.errorbar(f, counts, yerr=counts_err, marker='.', label='data %s' % label, linestyle='')
+        else:
+            plt.plot(np.arange(1,K+1), cumulative_sum - m4, label='data %s' % label)
+        #plt.plot(f, y, label='%d harmonics' % n_harm, linestyle='--')
+        plt.xlabel('Phase')
+        plt.legend()
+        if label != '':
+            plt.savefig('rms_%s.pdf' % label)
+    if verbose:
+        return PF_rms, max_harm
+    else:
+        return PF_rms
+
+def subtract_background(a0, background, background_error):
+    """subtracts background from the average
+
+    Args:
+        a0 (float): input average
+        background (numpy array): vector of background values
+        background_error (numpy array): vector of background values uncertainties
+
+    Raises:
+        Exception: if the subtracted continuun level is zero should we raise an exception ?
+
+    Returns:
+        a0 (loat): background subtracted average
+    """
+    ind = background_error > 0
+    if np.sum(ind)>0:
+        background_level = np.sum(background[ind]/background_error[ind]**2) / np.sum(1./background_error[ind]**2)
+        logger.debug('Background level %f' % background_level)
+        a0_out = a0 - background_level
+        if a0 == 0:
+            logger.warning('continuum level is zero')
+            #raise Exception
+    return a0_out
 
 def get_pulsed_fraction(e_min, e_max, pp, dpp, method_calc_rms='adaptive', output_file=None, force_recompute=False):
-    """ this function fits an input energy range of the pulse profile.
-        the fitting function will be a simple polynomial + gaussian
-        the aim is to retrieve basic gaussian parameters to be compared
-        to those obtained in spectral analysis around Ecycl.
+    """ thi functions is made to be a wrapper of different 
+    methods to compute the pulsed fraction
 
     Args:
         e_min (_type_): _description_
@@ -861,14 +1025,14 @@ def get_pulsed_fraction(e_min, e_max, pp, dpp, method_calc_rms='adaptive', outpu
             dpulsed_frac[i] = get_error_from_simul(x,dx, pulse_fraction_from_data_rms, n_harm=3)
         else:
         #RMS with adaptive n_harm
-            print(e_min[i], e_max[i])
+            # print(e_min[i], e_max[i])
             pulsed_frac[i] = fft_pulsed_fraction(x, dx, level=0.1, n_harm_min=2, n_harm_max=8, plot=False)
             dpulsed_frac[i] = get_error_from_simul(x,dx, fft_pulsed_fraction, level=0.1, n_harm_min=2, 
                                                          n_harm_max=8, verbose=False,  n_simul=300)
         
     if output_file is not None:
         with open(output_file, 'w') as ff:
-            ff.write("Energy Energy_error Pullsed_fraction Pulsed_fraction_Error\n")
+            ff.write("Energy Energy_error Pulsed_fraction Pulsed_fraction_Error\n")
             for i in range(0, len(e_min)):
                 ff.write("%.2f %.2f %.4f %.4f\n" % (ee_pulsed[i], dee_pulsed[i], 
                             pulsed_frac[i], dpulsed_frac[i]))
@@ -975,6 +1139,7 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
         amplitude=[[0,-1,1]]
         n_gauss=1
 
+    e_turn = 0
     if make_double_fit:
         if division_derivative_order == 2:
             e_turn = get_zeros(fake_en, smoothed_pulsed_frac_deriv_2, None, ind_range1)
@@ -1122,6 +1287,10 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
     ax1.set_ylim(y_lim)
     #ax1.set_xlim(x_lim)
     ax1.set_title(title)
+    if noFe is False:
+        ax1.axvline(6.5, 0,1, linestyle='--', color='cyan')
+    for x in forced_gaussian_centroids:
+        ax1.axvline(x, 0,1, linestyle='--', color='cyan')
 
     ax2.set_ylabel('Residuals')
     #ax1.set_ylim(-0.1, 1)
@@ -1129,11 +1298,13 @@ def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac,
         ax2.errorbar(ee_pulsed, np.concatenate([bb_low, bb_high]), xerr=dee_pulsed, yerr=1., fmt='.', color=col[2])
     else:
         ax2.errorbar(ee_pulsed, bb_low, xerr=dee_pulsed, yerr=1., fmt='.', color=col[2])
+
+        
     ax2.axhline(y=0, color='k', linestyle='--')
     if save_plot:
         plt.savefig(stem + 'global_pulsed_fitted.pdf')
 
-    return pulsed_fit_low, pulsed_fit_high
+    return pulsed_fit_low, pulsed_fit_high, e_turn
 
 def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center=[],
                     sigma=[], amplitude=[],
@@ -1252,6 +1423,8 @@ def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center
         ax1.set_ylabel('PF')
         ax1.set_ylim(y_lim)
         ax1.set_xlim(x_lim)
+        for N in range(1, n_gauss+1):
+            ax1.axvline(center[N - 1][0], 0, 1, linestyle='--', color='cyan')
         
         ax2.set_ylabel('Residuals')
         #ax1.set_ylim(-0.1, 1)
@@ -1305,7 +1478,7 @@ def explore_fit_mcmc(model_fitted, pars=[], hm_steps=7000, hm_walkers=2000, hm_j
     import corner
     if hm_steps <= hm_jump :
         logger.error(" You almost did it!. However, the number of steps must be higher than the jumped ones:"
-                     "hm_steps >>> hm_jump")
+                     "hm_steps > hm_jump")
 
     if pars ==[]:
         pars = model_fitted.params.copy()
@@ -1379,53 +1552,83 @@ def  error_from_simul_rms(counts, counts_err, n_simul=100, n_harm=3):
     return get_error_from_simul(counts, counts_err, pulse_fraction_from_data_rms,
                                 n_simul=n_simul, n_harms=n_harm)
 
-def get_error_from_simul(counts, counts_err, method, n_simul=100, use_poisson=False, **kwargs):
+def get_error_from_simul(counts, counts_err, method, n_simul=100, use_poisson=False, background=None,
+                         background_error=None, **kwargs):
     '''
-    :param counts: input data
-    :param counts_err: input data error
+    :param counts: vector of input data
+    :param counts_err: vector of input data errors
     :param method: a fnction with the method to compute the quantity
     :param n_simul: number of simulations
+    ;param background vector of background values
+    ;param background_error vector of background errors
     :param kwargs: additional arguments to the function to be computed
     :return:
     '''
+        
     simul_rms = []
+    simul_harm = []
+    fp_back = background
+    fp_back_error = background_error
     for i in range(n_simul):
         if use_poisson:
-            fp = random.poisson(counts)
+            fp = random.poisson(np.max( [np.ones(len(counts)), counts], axis=0))
+            if background is not None:
+                fp_back = random.poisson(np.max( [np.ones(len(background)), background], axis=0))
+                fp_back_error = np.sqrt(fp_back)
         else:
-            fp = random.normal(counts, counts_err)
-        simul_rms.append(method(fp, counts_err, **kwargs))
+            fp = random.normal(counts, np.max( [np.ones(len(counts_err)), counts_err], axis=0))
+            if background is not None and background_error is not None:
+                fp_back = random.normal(background, np.max( [np.ones(len(background_error)), background_error], axis=0))
+        if 'verbose' in kwargs and kwargs['verbose']:
+            x,y = method(fp, counts_err, background=fp_back, background_error=fp_back_error, **kwargs)
+            simul_rms.append(x)
+            simul_harm.append(y)
+        else:
+            simul_rms.append(method(fp, counts_err, **kwargs))
 
-    return np.std(simul_rms)
+    if 'plot' in kwargs and kwargs['plot']:
+        fig, axes = plt.subplots(1,2)
+        axes[0].hist(simul_rms, bins=10)
+        axes[1].hist(simul_harm, bins=10)
 
-def fft_pulsed_fraction(x, dx, level=0.1, n_harm_min=2, n_harm_max=-1, plot=False, verbose=True, label=''):
+    if 'verbose' in kwargs and kwargs['verbose']:
+        return np.std(simul_rms), np.max(simul_harm), np.min(simul_harm)
+    else:
+        return np.std(simul_rms)
+
+def fft_pulsed_fraction(x, dx, level=0.1, n_harm_min=2, n_harm_max=-1, plot=False, verbose=False, label='', 
+                        background=None, background_error=None):
     '''
     Computes the pulsed fraction using an FFT. It stops as soon as the pulse is described at better than level
     :param x: pulse profile
     :param dx: pulse profile uncertainty
-    :param level: confidenvce level for chi^2 test to stop number of harmonics (default 0.1)
+    :param level: confidence level for chi^2 test to stop number of harmonics (default 0.1)
     :param n_harm_min: minimum number of harmonics to use (default 2)
     :param n_harm_max: minimum number of harmonics to use (default -1 takes the size of pulse profile)
     :param plot: plot the pulse profile
     :param lavel: to save the plot with name "rms_`label`.pdf", if label=='' it does not save the plot
+    ;parma verbose (bool): if true it returns both pulsed_frac, n_harm, if false just the pulsed_frac
+    ;param background the vector of the background
+    ;param backgroun_error the uncertainty vector of the background
     :return: pulsed fraction (float)
     '''
     # import scipy.stats.chisquare as chi2
     from scipy.stats import chi2
     n = len(x)
 
-    if n_harm_max < n_harm_min or n_harm_max > n/2:
-        n_harm_max = n / 2
+    if n_harm_max < n_harm_min or n_harm_max+1 > n/2:
+        n_harm_max = n / 2 - 1
 
     fft = np.fft.fft(x)
     old_chi2_sf = -1.0
-    for n_harm in range(int(n_harm_min), int(n_harm_max)):
+    for n_harm in range(int(n_harm_min), int(n_harm_max)+1):
         mask = np.ones(n, dtype=np.cdouble)
         mask[n_harm:-(n_harm - 1)] = 0 + 0j
         ifft = np.fft.ifft(fft * mask)
         y = np.real(ifft)
         #Necessary to avoid infinite for zero uncertaity
         ind = dx > 0
+        #print(ind)
         chi2_val = np.sum(((x[ind] - y[ind]) / dx[ind]) ** 2)
         dof = max(1, n - (1 + 2 * (n_harm - 1)))
         chi2_sf = chi2.sf(chi2_val, dof)
@@ -1434,28 +1637,48 @@ def fft_pulsed_fraction(x, dx, level=0.1, n_harm_min=2, n_harm_max=-1, plot=Fals
         #Sometimes, we do ot reach the required level, but we cannot describe the pulse significantly better,
         # so we stop in any case (condition chi2_sf < old_chi2_sf)
         if chi2_sf > level or (chi2_sf < old_chi2_sf and chi2_sf > level/100.):
-            if verbose:
-                logger.info('chi2_sf = %e (level is %.2f) old_chi2_sf = %e' % (chi2_sf, level, old_chi2_sf))
+            logger.debug('chi2_sf = %e (level is %.2f) old_chi2_sf = %e' % (chi2_sf, level, old_chi2_sf))
             break
         old_chi2_sf = chi2_sf
 
-    if verbose:
-        logger.info("Used %d harmonics for pulse description" % n_harm)
+    logger.debug("Used %d harmonics for pulse description" % n_harm)
+    
     a = np.absolute(fft) / n
+
+    if background is not None and background_error is not None:
+        a[0] = subtract_background(a[0], background,background_error)
+    
     pulsed_frac = np.sqrt(np.sum(a[1:n_harm] ** 2) + np.sum(a[-n_harm + 1:] ** 2)) / a[0]
 
     if plot:
         import matplotlib.pyplot as plt
-        plt.figure()
+        #plt.figure()
         f = np.linspace(0, 1, n)
-        plt.errorbar(f, x, yerr=dx, marker='.', label='data %s' % label)
-        plt.plot(f, y, label='%d harmonics' % n_harm)
+        plt.errorbar(f, x, yerr=dx, marker='.', label='data %s' % label, linestyle='')
+        plt.plot(f, y, label='%d harmonics' % n_harm, linestyle='--')
         plt.xlabel('Phase')
         plt.legend()
         if label != '':
             plt.savefig('rms_%s.pdf' % label)
+    if verbose:
+        return pulsed_frac, n_harm
+    else:
+        return pulsed_frac
+
+def pf_rms_counts(x,dx, background=None, background_error=None):
+    '''
+    param x : pulse profile
+    param dx: error on pulse profile
+    return: pf_counts
+    '''
+    avg_cts=np.mean(x)
+    dev = (np.array(x)-avg_cts)**2
+    devi = np.sum(dev-np.array(dx)**2)
+    if background is not None and background_error is not None:
+        avg_cts = subtract_background(avg_cts, background, background_error)
+    pf_rms_cts = (1./ avg_cts) * np.sqrt( devi/len(x) )
+    return pf_rms_cts
 
-    return pulsed_frac
 
 def pulse_fraction_from_data_min_max(x,dx):
 
@@ -1647,7 +1870,7 @@ def get_fourier_coeff(pulse, n=2):
     phi = np.angle(ff[1:n+1])/2/np.pi
     return a, phi
 
-def get_fourier_coeff_error( counts, counts_err, n_simul=1000, use_poisson=False, n=2):
+def get_fourier_coeff_error( counts, counts_err, n_simul=1000, use_poisson=False, n=2, debug=False):
 
     sim_a = []
     sim_phi = []
@@ -1663,13 +1886,50 @@ def get_fourier_coeff_error( counts, counts_err, n_simul=1000, use_poisson=False
     err_a = np.zeros(n)
     err_phi = np.zeros(n)
     for i in range(n):
-        y = [x[i] for x in sim_a ]
+        y = np.array([x[i] for x in sim_a ])
         err_a[i] = np.std(y)
-        y = [x[i] for x in sim_phi ]
+        y = np.array([x[i] for x in sim_phi ])
+        y = align_phases(y, debug=debug)
         err_phi[i] = np.std(y)
         
     return err_a, err_phi
 
+def align_phases(y, nbins=20, debug=False, distance_factor=2):
+    hist,bins = np.histogram(y, bins=nbins)
+    from scipy.signal import find_peaks
+    peaks, _ = find_peaks(np.concatenate(([np.min(hist)],hist, [np.min(hist)])), distance=nbins/distance_factor)#, prominence=(0.5))
+    peaks-=1
+    if(debug):
+        plt.figure()
+        plt.hist(y, bins=nbins)
+
+        print(hist)
+        print(bins)
+        print(peaks)
+        
+    if (len(peaks) == 2):
+        ind_division = int((peaks[0]+peaks[1])/2)
+        if (peaks[0] > 0.25*nbins and peaks[0] < 0.75*nbins ) or (peaks[1] > 0.25*nbins and peaks[1] < 0.75*nbins ):
+            
+            logger.debug("No phase shift for two peaks")
+        else:
+            phase_division = bins[ind_division]
+            logger.debug("two peaks, phase_Division %f" % phase_division)
+            y[y>phase_division]-=1.
+    elif len(peaks) == 1:
+        if peaks[0] < nbins/4:
+            phase_division = bins[int(0.75*nbins)]
+            logger.debug("Phase_division" ,phase_division)
+            logger.debug(np.sum(y>phase_division))
+            y[y>phase_division]-=1
+        if peaks[0]>0.75*nbins:
+            phase_division = bins[int(0.25*nbins)]
+            logger.debug("Phase_division" ,phase_division)
+            logger.debug(np.sum(y>phase_division))
+            y[y<phase_division]+=1
+            
+    return y
+
 def rate_filter(time, rate, drate, level_down=0, level_up = 1e10, quantile=0.8, make_plot=True, 
                 far=1e-4, rate_label='rate', histo_bins=300):
     '''
@@ -1890,7 +2150,7 @@ def get_cross_correlation_with_error(pp, dpp, ee_pulsed=[], dee_pulsed=[], n_sim
 
         fake_lags[i, :], fake_corrs[i, :] = get_cross_correlation(fake_pp, n_to_fit=n_to_fit)
     
-    lag_errors = np.std(fake_lags, 0)
+    lag_errors = np.std(align_phases(fake_lags), 0)
     correlation_errors = np.std(fake_corrs, 0)
 
     if stem is not None:
